@@ -4,7 +4,7 @@ use 5.010;
 use strict;
 use warnings;
 use base 'Net::Server::PreFork';
-our $VERSION = '2.20181013'; # VERSION
+our $VERSION = '2.20181024'; # VERSION
 
 
 use English qw{ -no_match_vars };
@@ -52,6 +52,13 @@ sub get_installed_handlers {
 }
 
 
+sub idle_loop_hook {
+    my ( $self ) = @_;
+    $self->{'metric'}->master_metric_update( $self );
+    return;
+}
+
+
 sub pre_loop_hook {
     my ( $self ) = @_;
 
@@ -123,7 +130,6 @@ sub run_n_children_hook {
 
 sub child_init_hook {
     my ( $self ) = @_;
-
     srand();
 
     my $config = get_config();
@@ -194,8 +200,9 @@ sub child_finish_hook {
     my ($self) = @_;
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':exiting';
     $self->loginfo( "Child process $PID shutting down" );
-    $self->{'handler'}->{'_Handler'}->metric_count( 'reaped_children_total', {}, 1 );
-    $self->{'handler'}->{'_Handler'}->metric_send();
+    eval {
+        $self->{'handler'}->{'_Handler'}->metric_count( 'reaped_children_total', {}, 1 );
+    };
     $self->destroy_objects();
     return;
 }
@@ -269,41 +276,6 @@ sub get_client_details {
 }
 
 
-sub child_is_talking_hook {
-    my ( $self, $socket ) = @_;
-
-    my $request;
-    my $raw_request;
-
-    eval {
-        $raw_request = <$socket>;
-        $request = decode_json( $raw_request ) if defined $raw_request;
-    };
-    if ( my $error = $@ ) {
-        warn "Error $error reading from child";
-        return;
-    }
-    else {
-
-        return if ! defined $raw_request;
-
-        if ( ! $request ) {
-            warn "Ignoring Invalid child request: $raw_request;";
-            return;
-        }
-        $request =~ s/[\n\r]+$//;
-        if ( $request->{ 'method' } eq 'METRIC.GET' ) {
-            $self->{'metric'}->master_handler( $request, $socket, $self );
-        }
-        if ( $request->{ 'method' } eq 'METRIC.COUNT' ) {
-            $self->{'metric'}->master_handler( $request, $socket, $self );
-        }
-    }
-
-    return;
-}
-
-
 sub process_request {
     my ( $self ) = @_;
     my $config = $self->{'config'};
@@ -363,12 +335,9 @@ sub process_main {
     # Call close callback
     $self->{'handler'}->{'_Handler'}->top_close_callback();
     if ( $self->{'handler'}->{'_Handler'}->{'exit_on_close'} ) {
-        $self->{'metric'}->send( $self );
         $self->fatal('exit_on_close requested');
     }
 
-    $self->{'metric'}->send( $self );
-
     if ( $config->{'debug'} ) {
         my $process_table = Proc::ProcessTable->new();
         foreach my $process ( @{$process_table->table} ) {
@@ -693,7 +662,7 @@ sub start {
                 'port'  => $path,
                 'proto' => 'unix',
             };
-            $srvargs{'child_communication'} = 1;
+            $srvargs{'child_communication'} = 0;
 
             if ($umask) {
                 umask ( oct( $umask ) );
@@ -796,7 +765,6 @@ sub setup_handlers {
     $self->{'handler'}->{'_Handler'} = $handler;
 
     $handler->metric_count( 'forked_children_total', {}, 1 );
-    $handler->metric_send();
 
     my $config = $self->{'config'};
     foreach my $name ( @{$config->{'load_handlers'}} ) {
@@ -1067,7 +1035,7 @@ Mail::Milter::Authentication - A Perl Mail Authentication Milter
 
 =head1 VERSION
 
-version 2.20181013
+version 2.20181024
 
 =head1 SYNOPSIS
 
@@ -1089,6 +1057,10 @@ A Perl Implementation of email authentication standards rolled up into a single
 
 =head1 METHODS
 
+=head2 I<idle_loop_hook()>
+
+Hook which runs in the master periodically.
+
 =head2 I<pre_loop_hook()>
 
 Hook which runs in the master before looping.
@@ -1129,10 +1101,6 @@ Get the path of the connecting client.
 
 Get the details of the connecting client.
 
-=head2 I<child_is_talking_hook( $sock )>
-
-Hook which runs when a child wishes to communicate with the parent.
-
 =head2 I<process_request()>
 
 Hook which runs for each request, passes control to metrics handler or process_main as appropriate.
