@@ -1,7 +1,7 @@
 package Mail::Milter::Authentication::Handler;
 use strict;
 use warnings;
-our $VERSION = '2.20181013'; # VERSION
+our $VERSION = '2.20181024'; # VERSION
 
 
 use Digest::MD5 qw{ md5_hex };
@@ -85,7 +85,8 @@ sub metric_count {
 
 sub metric_send {
     my ( $self ) = @_;
-    $self->{'thischild'}->{'metric'}->send( $self->{ 'thischild' });
+    # NOOP
+    # TODO Deprecate and remove
     return;
 }
 
@@ -119,7 +120,7 @@ sub rbl_check_domain {
     if ($packet) {
         foreach my $rr ( $packet->answer ) {
             if (  lc $rr->type eq 'a' ) {
-                return 1;
+                return $rr->address();
             }
         }
     }
@@ -2014,7 +2015,7 @@ Mail::Milter::Authentication::Handler
 
 =head1 VERSION
 
-version 2.20181013
+version 2.20181024
 
 =head1 DESCRIPTION
 
