@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Handler::DMARC;
 use strict;
 use warnings;
 use base 'Mail::Milter::Authentication::Handler';
-our $VERSION = '2.20181013'; # VERSION
+our $VERSION = '2.20181024'; # VERSION
 
 use Data::Dumper;
 use English qw{ -no_match_vars };
@@ -536,14 +536,8 @@ sub _process_dmarc_for {
     if ($rua) {
         if ( ! $config->{'no_report'} ) {
             if ( ! $self->{'skip_report'} ) {
-                eval {
-                    $self->dbgout( 'DMARCReportTo', $rua, LOG_INFO );
-                    $dmarc->save_aggregate();
-                };
-                if ( my $error = $@ ) {
-                    $self->handle_exception( $error );
-                    $self->log_error( 'DMARC Report Error ' . $error );
-                }
+                $self->dbgout( 'DMARCReportTo', $rua, LOG_INFO );
+                push @{ $self->{'report_queue'} }, $dmarc;
             }
             else {
                 $self->dbgout( 'DMARCReportTo (skipped flag)', $rua, LOG_INFO );
@@ -605,6 +599,7 @@ sub new_dmarc_object {
 sub helo_callback {
     my ( $self, $helo_host ) = @_;
     $self->{'helo_name'} = $helo_host;
+    $self->{'report_queue'} = [] if ! $self->{'report_queue'};
     return;
 }
 
@@ -837,8 +832,47 @@ sub _add_dmarc_header {
     return;
 }
 
+sub addheader_callback {
+    my $self = shift;
+    my $handler = shift;
+    return;
+}
+
+sub _save_aggregate_reports {
+    my ( $self ) = @_;
+    return if ! $self->{'report_queue'};
+    # Try as best we can to save a report, but don't stress if it fails.
+    eval {
+        $self->set_handler_alarm( 2 * 1000000 ); # Allow no longer than 2 seconds for this!
+        while ( my $report = shift @{ $self->{'report_queue'} } ) {
+            $report->save_aggregate();
+            $self->dbgout( 'DMARC Report saved for', $report->result()->published()->rua(), LOG_INFO );
+        }
+        $self->reset_alarm();
+    };
+    if ( my $Error = $@ ) {
+        $self->reset_alarm();
+        my $Type = $self->is_exception_type( $Error );
+        if ( $Type ) {
+            if ( $Type eq 'Timeout' ) {
+                # We have a timeout, is it global or is it ours?
+                if ( $self->get_time_remaining() > 0 ) {
+                    # We have time left, but the aggregate save timed out
+                    # Log this and move on!
+                    $self->log_error( 'DMARC timeout saving reports' );
+                    return;
+                }
+            }
+        }
+        $self->handle_exception( $Error );
+        $self->log_error( 'DMARC Report Error ' . $Error );
+    }
+    return;
+}
+
 sub close_callback {
     my ( $self ) = @_;
+    $self->_save_aggregate_reports();
     delete $self->{'helo_name'};
     delete $self->{'env_from'};
     delete $self->{'env_to'};
@@ -847,6 +881,7 @@ sub close_callback {
     delete $self->{'is_list'};
     delete $self->{'from_header'};
     delete $self->{'from_heades'};
+    delete $self->{'report_queue'};
     $self->destroy_object('dmarc');
     $self->destroy_object('dmarc_result');
     return;
@@ -866,7 +901,7 @@ Mail::Milter::Authentication::Handler::DMARC
 
 =head1 VERSION
 
-version 2.20181013
+version 2.20181024
 
 =head1 DESCRIPTION
 
