@@ -1,7 +1,7 @@
 package Mail::Milter::Authentication::Tester;
 use strict;
 use warnings;
-our $VERSION = '2.20181013'; # VERSION
+our $VERSION = '2.20181024'; # VERSION
 
 our @ISA = qw{ Exporter }; ## no critic
 our @EXPORT = qw{ start_milter stop_milter get_metrics test_metrics smtp_process smtp_process_multi milter_process smtpput send_smtp_packet smtpcat }; ## no critic
@@ -139,6 +139,9 @@ sub test_metrics {
                 elsif ( $key =~ /authmilter_forked_children_total/ ) {
                     is( $metrics->{ $key } > 0, $data->{ $key } > 0, "Metrics $key" );
                 }
+                elsif ( $key =~ /authmilter_processes_/) {
+                    is( $metrics->{ $key } > -1, $data->{ $key } > -1, "Metrics $key" );
+                }
                 else {
                     is( $metrics->{ $key }, $data->{ $key }, "Metrics $key" );
                 }
@@ -160,6 +163,9 @@ sub test_metrics {
                 elsif ( $key =~ /authmilter_forked_children_total/ ) {
                     $metrics->{ $key } = 123456 if $metrics->{ $key } > 0;
                 }
+                elsif ( $key =~ /authmilter_processes_/) {
+                    $metrics->{ $key } = 123456 if $metrics->{ $key } > -1;
+                }
             }
             open my $OutF, '>', $expected;
             $j->pretty();
@@ -579,7 +585,7 @@ Mail::Milter::Authentication::Tester
 
 =head1 VERSION
 
-version 2.20181013
+version 2.20181024
 
 =head1 AUTHOR
 
