@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Tester::HandlerTester;
 
 use strict;
 use warnings;
-our $VERSION = '2.20181013'; # VERSION
+our $VERSION = '2.20181024'; # VERSION
 
 use Carp;
 use Clone qw{ clone };
@@ -38,8 +38,8 @@ sub _build_config_smtp {
         'tempfail_on_error_local'         => 1,
         'tempfail_on_error_trusted'       => 1,
 
-        'metric_connection'               => 'unix:tmp/authentication_milter_test_metrics.sock',
-        'metric_umask'                    => '0000',
+        '#metric_connection'               => 'unix:tmp/authentication_milter_test_metrics.sock',
+        '#metric_umask'                    => '0000',
 
         'protocol' => 'smtp',
         'smtp' => {
@@ -75,8 +75,8 @@ sub _build_config_milter {
         'tempfail_on_error_local'         => 1,
         'tempfail_on_error_trusted'       => 1,
 
-        'metric_connection'               => 'unix:tmp/authentication_milter_test_metrics.sock',
-        'metric_umask'                    => '0000',
+        '#metric_connection'               => 'unix:tmp/authentication_milter_test_metrics.sock',
+        '#metric_umask'                    => '0000',
 
         'protocol' => 'milter',
 
@@ -366,7 +366,7 @@ Mail::Milter::Authentication::Tester::HandlerTester
 
 =head1 VERSION
 
-version 2.20181013
+version 2.20181024
 
 =head1 SYNOPSIS
 
